library(igraph)
library(scales)

shinyServer(
  function(input, output) {

    # read and cache the graph
    inputGraph = reactive({
      inFile = input$graph
      if (!is.null(inFile))
        read.graph(file=inFile$datapath, format=input$format)
    })

    # render scatterplot
    output$scatter = renderPlot({
      # viualize button
      input$compare
      # isolate input graph
      g = isolate(inputGraph())
      
      if (!is.null(g)) {      
        # isolate centrality measures
        x = isolate(switch(input$centrality1, 
                      "degree" = degree(g, mode = input$degree1),
                      "eigenvector" = evcent(g)$vector,
                      "Katz" = alpha.centrality(g, alpha = input$alpha1),
                      "pagerank" = page.rank(g, damping = input$alpha1)$vector,
                      "HITS" = if (input$HITS1 == "authority") authority.score(g)$vector else hub.score(g)$vector,
                      "closeness" = closeness(g, mode="out"),
                      "betweenness" = betweenness(g)))
        
        y = isolate(switch(input$centrality2, 
                           "degree" = degree(g, mode = input$degree2),
                           "eigenvector" = evcent(g)$vector,
                           "Katz" = alpha.centrality(g, alpha = input$alpha2),
                           "pagerank" = page.rank(g, damping = input$alpha2)$vector,
                           "HITS" = if (input$HITS2 == "authority") authority.score(g)$vector else hub.score(g)$vector,
                           "closeness" = closeness(g, mode="out"),
                           "betweenness" = betweenness(g)))
      
        xlab = isolate(input$centrality1)
        ylab = isolate(input$centrality2)
        # Tufte style
        plot(x, y, xlab = xlab, ylab = ylab, xlim=c(min(x), max(x)), ylim=c(min(y), max(y)), 
             bty="n", xaxt='n', yaxt='n')
        axis(side=1, at=round(c(min(x), median(x), mean(x), max(x)), 3), las="2")
        axis(side=2, at=round(c(min(y), median(y), mean(y), max(y)), 3), las="2")
        rug(x, side=1, col="red")
        rug(y, side=2, col="red")
        r = round(cor(x,y), 2)
        legend = paste("r =", r)
        legend(x = "bottomright", legend = legend, bty="n")

      }      
    })
    
    
  }
)